import time
from time import sleep

import busypie
from busypie.durations import SECOND, ONE_HUNDRED_MILLISECONDS

_DEFAULT_MAX_WAIT_TIME = 10 * SECOND


class ConditionBuilder:

    def __init__(self):
        self._condition = Condition()

    def at_most(self, value, unit=SECOND):
        self._condition.wait_time_in_secs = value * unit
        return self

    def ignore_exceptions(self):
        self._condition.ignore_all_exceptions = True
        return self

    def until(self, func):
        ConditionAwaiter(self._condition).wait_for(func)


class Condition:
    wait_time_in_secs = _DEFAULT_MAX_WAIT_TIME
    ignore_all_exceptions = False


class ConditionAwaiter:

    def __init__(self, condition):
        self._condition = condition

    def wait_for(self, func):
        start_time = time.time()
        while True:
            try:
                if func():
                    break
            except Exception as e:
                self._raise_exception_if_needed(e)
            self.validate_wait_constraint(func.__name__, start_time)
            sleep(ONE_HUNDRED_MILLISECONDS)

    def _raise_exception_if_needed(self, e):
        if not self._condition.ignore_all_exceptions:
            raise e

    def validate_wait_constraint(self, func_name, start_time):
        if (time.time() - start_time) > self._condition.wait_time_in_secs:
            raise busypie.ConditionTimeoutError("Failed to meet condition of {} within {} seconds"
                                        .format(func_name, self._condition.wait_time_in_secs))


class ConditionTimeoutError(Exception):
    pass



from busypie.durations import SECOND
from busypie.condition import ConditionBuilder


def wait():
    return ConditionBuilder()


def wait_at_most(value, unit=SECOND):
    return wait().at_most(value, unit)



from .core import wait, wait_at_most
from .condition import ConditionTimeoutError
from .durations import MILLISECOND, SECOND, MINUTE, HOUR, \
    ONE_HUNDRED_MILLISECONDS, FIVE_HUNDRED_MILLISECONDS, \
    ONE_SECOND, FIVE_SECONDS, TEN_SECONDS, \
    ONE_MINUTE, FIVE_MINUTES, TEN_MINUTES

__all__ = [
    'wait', 'wait_at_most',
    'ConditionTimeoutError',

    # Durations
    'MILLISECOND', 'SECOND', 'MINUTE', 'HOUR',
    'ONE_HUNDRED_MILLISECONDS', 'FIVE_HUNDRED_MILLISECONDS',
    'ONE_SECOND', 'FIVE_SECONDS', 'TEN_SECONDS',
    'ONE_MINUTE', 'FIVE_MINUTES', 'TEN_MINUTES'
]




# Time definitions in seconds
MILLISECOND = 0.001
SECOND = 1
MINUTE = 60 * SECOND
HOUR = 60 * MINUTE

# Helpful times in seconds
ONE_HUNDRED_MILLISECONDS = 100 * MILLISECOND
FIVE_HUNDRED_MILLISECONDS = 500 * MILLISECOND
ONE_SECOND = SECOND
TWO_SECONDS = 2 * SECOND
FIVE_SECONDS = 5 * SECOND
TEN_SECONDS = 10 * SECOND
ONE_MINUTE = MINUTE
TWO_MINUTES = 2 * MINUTE
FIVE_MINUTES = 5 * MINUTE
TEN_MINUTES = 10 * MINUTE



import os

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    readme = fh.read()

setup(
    name="busypie",
    version="0.1.1",
    author="Eli Segal",
    author_email="eli.segal@gmail.com",
    license='Apache License 2.0',
    description="Busy wait easy-peasy",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/rockem/busypie",
    packages=['busypie'],
    setup_requires=["pytest-runner"],
    install_require=[

    ],
    tests_require=[
        'pytest==5.3.2',
        'pytest-timeout==1.3.3'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)